function check_basics_motions_s(Mss,Msps,b_output)
%%  check basic relations for motions with exponential repr.
%
% twist vector of adjoint motion
% twist vector of inverse motion
% function for inverse motion
% twist vector of concatenated motion
% twist vector of relative motion
% difference motion M12 as concatenated of inv(M1) and M2
%
% wf 6/2020

% for generating small twist vectors
sigma_n = 10^-8;
% single motion and pair of motions
Ms  = Mss.Ms;
Msp = Msps.Msp;

% adjoint motion matrix of M
Mad      = [Ms(1:3,1:3) zeros(3); calc_S(Ms(1:3,4))*Ms(1:3,1:3) Ms(1:3,1:3)];

%% adjoint
% small twist and motion
s      = randn(6,1)*sigma_n;
M_s    = expm(calc_A_from_s(s));

% adjoint motion
M_s_ad = Ms * M_s * inv(Ms);

% twist of adjoint motion
s_ad   = calc_s_from_A(M_s_ad);

% check
check_s_ad = (s_ad - Mad * s)';
if b_output
    check_s_ad = (s_ad - Mad * s)'
end

%% inverse

% noisy motion and inverse
M_s_r   = M_s * Ms;
M_s_inv = inv(M_s_r);

% twist of inverse
s_inv   = calc_s_from_A(M_s_inv * Ms);

% check
check_s_inv = (s_inv -(- inv(Mad) * s))';
if b_output
    check_s_inv = (s_inv -(- inv(Mad) * s))'
end

% check forward and backward M = inv(inv(M))
[Mssi,J] = calc_inverse_M_s(Mss);
MM       = calc_inverse_M_s(Mssi);

% check mean and CovM
check_Ms = Mss.Ms-MM.Ms;
check_Cs = Mss.Cs-MM.Cs;
check_J = J-(-inv(Mad));
if b_output
    check_Ms = Mss.Ms-MM.Ms
    check_Cs = Mss.Cs-MM.Cs
    check_J = J-(-inv(Mad));
end

%% concatenation M = M2*M1

% joint noisy twist vectors
s   = randn(12,1)*sigma_n;
% noisy motions
M1_r = expm(calc_A_from_s(s(1:6 )))*Msp(:,1:4);
M2_r = expm(calc_A_from_s(s(7:12)))*Msp(:,5:8);
% true and noisy concatenation
M    = Msp(:,5:8) * Msp(:,1:4);
M_r  = M2_r * M1_r;
% twist of noise of concatenations
s_e  = calc_s_from_A(M_r*inv(M));

% adjunct motion matrix of M2
R2     = Msp(1:3,5:7);
Z2     = Msp(1:3,8);
M_2_ad = [R2 zeros(3); calc_S(Z2)*R2 R2];

% check
check_s_concatenate = (s_e - (M_2_ad*s(1:6) + s(7:12)))';
if b_output
    check_s_concatenate = (s_e - (M_2_ad*s(1:6) + s(7:12)))'
end


%% relative motion M = inv(M1)*M2

% joint noisy twist vectors
s    = randn(12,1)*sigma_n;
% noisy motions
M1_r = expm(calc_A_from_s(s(1:6 )))*Msp(:,1:4);
M2_r = expm(calc_A_from_s(s(7:12)))*Msp(:,5:8);
% true and noisy relative motion
M    = inv(Msp(:,1:4)) * Msp(:,5:8) ;
M_r  = inv(M1_r)      * M2_r;
% twist of noise of relative motion
s_e  = calc_s_from_A(M_r*inv(M));

% adjunct motion matrix of M1
R1     = Msp(1:3,1:3);
Z1     = Msp(1:3,4);
M_1_ad = [R1 zeros(3); calc_S(Z1)*R1 R1];

% check
check_s_relative = (s_e - (inv(M_1_ad)*(s(7:12) - s(1:6))))';
if b_output
    check_s_relative = (s_e - (inv(M_1_ad)*(s(7:12) - s(1:6))))'
end
%% check all three functions, assume independence
% M1 and M2
Ms1.Ms   = Msps.Msp(:,1:4);
Ms2.Ms   = Msps.Msp(:,5:8);
% joint CovM, uncorrelated motions
C        = [Msps.Csp(1:6,1:6) zeros(6); zeros(6) Msps.Csp(7:12,7:12)];
C1       = Msps.Csp(1:6,1:6);
C2       = Msps.Csp(7:12,7:12);
% motion pair
Msps.Csp = C;
% individual motions
Ms1.Cs   = C1;
Ms2.Cs   = C2;

% relative motion Md = M1' * M2
Msd     = calc_relative_M_s(Msps);
% check with concatenation:
% inverse of M1
Ms1i    = calc_inverse_M_s(Ms1);
% concatenate 
Ms12.Msp = [Ms2.Ms,Ms1i.Ms];
Ms12.Csp = [Ms2.Cs zeros(6); zeros(6) Ms1i.Cs];
Msd_ch   = calc_concatenated_M_s(Ms12);

% check mean and CovM of rel(M1,M2) = con(M2,inv(M1))
check_relative_Ms = Msd_ch.Ms-Msd.Ms;
check_relative_Cs = Msd_ch.Cs-Msd.Cs;

if b_output
    check_relative_Ms = Msd_ch.Ms-Msd.Ms
    check_relative_Cs = Msd_ch.Cs-Msd.Cs
end

%% final evaluation
T = 10^-6;
if norm(check_s_ad)       < T*sigma_n && ...
        norm(check_s_inv) < T*sigma_n && ...
        norm(check_Ms)     < T*sigma_n && ...
        norm(check_Cs)     < T*sigma_n^2 && ...
        norm(check_J)      < T*sigma_n && ...
        norm(check_s_concatenate) < T*sigma_n && ...
        norm(check_s_relative)    < T*sigma_n && ...
        norm(check_relative_Ms)    < T*sigma_n && ...
        norm(check_relative_Cs)    < T*sigma_n^2
    
    display('motions s        ok +++++++++++++++++++++++++++++++++++++++')
else
    display('notions s        not ok ***********************************')
end
end

