function check_basics_rotations(Rs,Rps,b_output)
%%  check basic relations for rotations
%
% rotation vector of adjoint rotation
% rotation vector of inverse rotation
% function for inverse rotation
% rotation vector of concatenated rotation
% rotation vector of relative rotation
% difference rotation M12 as concatenated of inv(R1) and R2
%
% wf 5/2020

% for generating small twist vectors
sigma_n = 10^-8;
% basic rotation and rotation pair
R = Rs.R;
Rp = Rps.Rp;

%% adjoint rotation
% small rotation vector
r = randn(3,1)*sigma_n;
% adjoint rotation
R_r_ad = R * expm(calc_S(r)) * R';
% rotation vector of adjoint rotation
r_ad  = calc_v_from_S(R_r_ad);
%check
check_rad = (r_ad - R * r)';
if b_output
    check_rad = (r_ad - R * r)'
end

%% inverse
% small rotation vector
r   = randn(3,1)*sigma_n;
% noisy rotation
R_r = expm(calc_S(r))*R;
% inverse
R_inv_r = R_r';
% rotation vector of noise of inverse 
r_inv  = calc_v_from_S(R_inv_r * R);

% check
    check_r_inv = (r_inv -(- R' * r))';
if b_output
    check_r_inv = (r_inv -(- R' * r))'
end

% check forward and backward of function inverse_rotation: R = inv(inv(R))
[Rsi,J]=calc_inverse_R(Rs);
RR =calc_inverse_R(Rsi);

%check
    check_R = Rs.R-RR.R;
    check_C = Rs.C-RR.C;
    check_J = J-(-R');
if b_output
    check_R = Rs.R-RR.R
    check_C = Rs.C-RR.C
    check_J = J-(-R')
end

%% concatenation R = R2*R1

% rotation vectors of noise of rotations 
r    = randn(6,1)*sigma_n;
% noisy rotations
R1_r = expm(calc_S(r(1:3)))*Rp(:,1:3);
R2_r = expm(calc_S(r(4:6)))*Rp(:,4:6);
% true and noisy concatenation
R    = Rp(:,4:6) * Rp(:,1:3);
R_r  = R2_r * R1_r;
% rotation vector of noise of concatenated rotations
r_e  = calc_v_from_S(R_r*R');

% check
    check_r_concatenate = r_e - (Rp(:,4:6)*r(1:3) + r(4:6));
if b_output
    check_r_concatenate = r_e - (Rp(:,4:6)*r(1:3) + r(4:6))
end

%% difference rotation R = inv(R1)*R2

% rotation vectors of noise of rotations 
r    = randn(6,1)*sigma_n;
% noisy rotations
R1_r = expm(calc_S(r(1:3)))*Rp(:,1:3);
R2_r = expm(calc_S(r(4:6)))*Rp(:,4:6);
% true and noisy concatenation
R    = Rp(:,1:3)' * Rp(:,4:6) ;
R_r  = R1_r'      * R2_r;
% rotation vector of noise of relative rotation
r_e  = calc_v_from_S(R_r*R');

%check
    check_r_relative = r_e - (Rp(:,1:3)'*(r(4:6) - r(1:3)));
if b_output
    check_r_relative = r_e - (Rp(:,1:3)'*(r(4:6) - r(1:3)))
end

%% check all three functions, assume independence rel(R1,R2) = con(R2,inv(R1))
R1.R  = Rps.Rp(:,1:3);
R2.R  = Rps.Rp(:,4:6);
C     = [Rps.Cp(1:3,1:3) zeros(3); zeros(3) Rps.Cp(4:6,4:6)];
C1    = Rps.Cp(1:3,1:3);
C2    = Rps.Cp(4:6,4:6);
Rps.Cp = C;
R1.C  = C1;
R2.C  = C2;

% relative rotation Rd = R1' * R2
Rd     = calc_relative_R(Rps);
% check with concatenation:
% inverse of R1
R1i    = calc_inverse_R(R1);
% concatenate 
R12.Rp = [R2.R,R1i.R];
R12.Cp = [R2.C zeros(3); zeros(3) R1i.C];
Rd_ch  = calc_concatenated_R(R12);

% check
    check_relative_R = Rd_ch.R-Rd.R;
    check_relative_C = Rd_ch.C-Rd.C;
if b_output
    check_relative_R = Rd_ch.R-Rd.R
    check_relative_C = Rd_ch.C-Rd.C
end


% total evaluation
T = 10^-6;
if norm(check_rad)   < T*sigma_n && ...
   norm(check_r_inv) < T*sigma_n && ...
   norm(check_R)     < T*sigma_n && ...
   norm(check_C)     < T*sigma_n^2 && ...
   norm(check_J)     < T*sigma_n && ...
   norm(check_r_concatenate) < T*sigma_n && ...
   norm(check_r_relative)    < T*sigma_n && ...
   norm(check_relative_R)    < T*sigma_n && ...
   norm(check_relative_C)    < T*sigma_n^2 
   
    display('rotations         ok       +++++++++++++++++++++++++++++++++')
else
    display('rotations         not ok   *********************************')
end

