%% demo uncertain frames: rotations, motion(s), motions(z)
%
% 1. Jacobians for adjoint, inverse, concatenation and difference
% 2. Generate and statistically test uncertain frames 
%
% Basic representations as structs
%
% rotations: R.R = [3x3] and R.C = [3x3]
% motions 
%     in exponential (s) and partially exponential (z) representation   
%     Ms.Ms and Mz.Mz = [4x4] and Ms.Cs and Mz.Cz = [6x6]
% 
% rotation pairs
%     Rp.Rp = [R1,R2]=[3x6] and Rp.Cp = [6x6]
% motion pairs:   
%     Msp.Msp and Mzp.Mzp = [M1,M2]=[4x8] and Msp.Csp and Mzp.Czp = [12x12]
%
% wf 05/2020
%

clearvars
close all
clc


display('----------------------------------------------------------')
display('----------------- demo uncertain frames  -----------------')
display('----------------------------------------------------------')

%% control parameters 

init_rand = 0;              % start seed for random numbers
init_rand_seed(init_rand);

S         = 0.999;          % significance level

b_output  = 1;              % short or long output
%b_output  = 0;

sigma_g   = 1;              % for generating frames

sigma_n   = 10^-6;          % noise component

N_samples = 100;             % number of samples for testing 

%% generate data
[R0p,M0p,Cp,R,Ms,Mz,Rp,Msp,Mzp,Rs,Mss,Mzs]=...
    generate_test_data_uncertain_frames(sigma_g,sigma_n,N_samples);
   %generate_test_data_uncertain_frames
   %generate_test_data_uncertain_frames


%% checks
check_basics_rotations(R,Rp,b_output)

check_basics_motions_s(Ms,Msp,b_output)

check_basics_motions_z(Mz,Mzp,b_output)

display(' ')

check_simulated_rotation(R0p(1:3,1:3),Cp(1:3,1:3),Rs,S);

check_simulated_motion_s(M0p(1:4,1:4),Cp(1:6,1:6),Mss,S)

check_simulated_motion_z(M0p(1:4,1:4),Cp(1:6,1:6),Mzs,S)

display(' ')
display('----------------------------------------------------------')
display('--------------- end demo uncertain frames  ---------------')
display('----------------------------------------------------------')

return


 

